#!/usr/bin/env python3

import os
import subprocess
import sys
import shutil
import typing as t

def main():
    if len(sys.argv) < 4 or '--' not in sys.argv:
        print("Usage: remove-before-wrapper <output> -- <nix command...>")
        sys.exit(1)

    # Extract the parts
    output: str = sys.argv[1]
    nix_command_idx: int = sys.argv.index('--') + 1
    nix_command: t.List[str] = sys.argv[nix_command_idx:]

    output_temp: str = output + '.tmp'

    # Remove the output and temp output in case they exist
    shutil.rmtree(output, ignore_errors=True)
    shutil.rmtree(output_temp, ignore_errors=True)

    # Execute nix command with `--write-to` tempary output
    nix_command_write_to = nix_command + ['--write-to', output_temp]
    subprocess.run(nix_command_write_to, check=True)

    # Move the temporary output to the intended location
    os.rename(output_temp, output)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import glob
import sys

# meson expects makefile-style dependency declarations, i.e.
#
#   target: dependency...
#
# meson seems to pass depfiles straight on to ninja even though
# it also parses the file itself (or at least has code to do so
# in its tree), so we must live by ninja's rules: only slashes,
# spaces and octothorpes can be escaped, anything else is taken
# literally. since the rules for these aren't even the same for
# all three we will just fail when we encounter any of them (if
# asserts are off for some reason the depfile will likely point
# to nonexistant paths, making everything phony and thus fine.)
for path in glob.glob(sys.argv[1] + '/**', recursive=True):
    assert '\\' not in path
    assert ' ' not in path
    assert '#' not in path
    print("ignored:", path)



#!/usr/bin/env python3

from pathlib import Path
import json
import os, os.path
import sys
import typing as t

name = 'substitute.py'

def log(*args: t.Any, **kwargs: t.Any) -> None:
    kwargs['file'] = sys.stderr
    print(f'{name}:', *args, **kwargs)

def do_include(content: str, relative_md_path: Path, source_root: Path, search_path: Path) -> str:
    assert not relative_md_path.is_absolute(), f'{relative_md_path=} from mdbook should be relative'

    md_path_abs = source_root / relative_md_path
    var_abs = md_path_abs.parent
    assert var_abs.is_dir(), f'supposed directory {var_abs} is not a directory (cwd={os.getcwd()})'

    lines = []
    for l in content.splitlines(keepends=True):
        if l.strip().startswith("{{#include "):
            requested = l.strip()[11:][:-2]
            if requested.startswith("@generated@/"):
                included = search_path / Path(requested[12:])
                requested = included.relative_to(search_path)
            else:
                included = source_root / relative_md_path.parent / requested
                requested = included.resolve().relative_to(source_root)
            assert included.exists(), f"{requested} not found at {included}"
            lines.append(do_include(included.read_text(), requested, source_root, search_path) + "\n")
        else:
            lines.append(l)
    return "".join(lines)

def recursive_replace(data: dict[str, t.Any], book_root: Path, search_path: Path) -> dict[str, t.Any]:
    match data:
        case {'sections': sections}:
            return data | dict(
                sections = [recursive_replace(section, book_root, search_path) for section in sections],
            )
        case {'Chapter': chapter}:
            path_to_chapter = Path(chapter['path'])
            chapter_content = chapter['content']

            return data | dict(
                Chapter = chapter | dict(
                    # first process includes. this must happen before docroot processing since
                    # mdbook does not see these included files, only the final agglomeration.
                    content = do_include(
                        chapter_content,
                        path_to_chapter,
                        book_root,
                        search_path
                    ).replace(
                        '@docroot@',
                        ("../" * len(path_to_chapter.parent.parts) or "./")[:-1]
                    ),
                    sub_items = [
                        recursive_replace(sub_item, book_root, search_path)
                        for sub_item in chapter['sub_items']
                    ],
                ),
            )

        case rest:
            assert False, f'should have been called on a dict, not {type(rest)=}\n\t{rest=}'

def main() -> None:


    if len(sys.argv) > 1 and sys.argv[1] == 'supports':
        return 0

    # includes pointing into @generated@ will look here
    search_path = Path(os.environ['MDBOOK_SUBSTITUTE_SEARCH'])

    if len(sys.argv) > 1 and sys.argv[1] == 'summary':
        print(do_include(
                        sys.stdin.read(),
                        Path('source/SUMMARY.md'),
                        Path(sys.argv[2]).resolve(),
                        search_path))
        return

    # mdbook communicates with us over stdin and stdout.
    # It splorks us a JSON array, the first element describing the context,
    # the second element describing the book itself,
    # and then expects us to send it the modified book JSON over stdout.

    context, book = json.load(sys.stdin)

    # book_root is the directory where book contents leave (ie, source/)
    book_root = Path(context['root']) / context['config']['book']['src']

    # Find @var@ in all parts of our recursive book structure.
    replaced_content = recursive_replace(book, book_root, search_path)

    replaced_content_str = json.dumps(replaced_content)

    # Give mdbook our changes.
    print(replaced_content_str)

try:
    sys.exit(main())
except AssertionError as e:
    print(f'{name}: INTERNAL ERROR in mdbook preprocessor: {e}', file=sys.stderr)
    print(f'this is a bug in {name}', file=sys.stderr)
    raise



#!/usr/bin/env nix-shell
#!nix-shell -i python3 -p python3 --pure

# To be used with `--trace-function-calls` and `flamegraph.pl`.
#
# For example:
#
# nix-instantiate --trace-function-calls '<nixpkgs>' -A hello 2> nix-function-calls.trace
# ./contrib/stack-collapse.py nix-function-calls.trace > nix-function-calls.folded
# nix-shell -p flamegraph --run "flamegraph.pl nix-function-calls.folded > nix-function-calls.svg"

import sys
from pprint import pprint
import fileinput

stack = []
timestack = []

for line in fileinput.input():
    components = line.strip().split(" ", 2)
    if components[0] != "function-trace":
        continue

    direction = components[1]
    components = components[2].rsplit(" ", 2)

    loc = components[0]
    _at = components[1]
    time = int(components[2])

    if direction == "entered":
        stack.append(loc)
        timestack.append(time)
    elif direction == "exited":
        dur = time - timestack.pop()
        vst = ";".join(stack)
        print(f"{vst} {dur}")
        stack.pop()
